c******************************************************************
c
c   begin function FL_VVgg
c
c*****************************************************************
c
c      Robin Roth  <robin@particle.uni-karlsruhe.de>
c	Initial version:  2012 December
c	Last modified:  2012 December
c       
c  fill hepup color and flavor assignments for WBF processes with
c  one attached external gluon and count subprocesses. 
c
c  WARNING!!      
c  Color is not yet treated correctly
c  colstruc = -3 is used
c
c
c
c  from FL_VVg:      
c
c  There are 2 color structures depending into which fermion line the 
c  gluon is inserted
c
c  colstruc = 2:upper
c             3:lower fermion line
c
c  cases can be distinguished according to which iflav(i)=0, i.e. corresponds
c  to the gluon

c  id1,2 are flavor identifiers for incoming quarks
c  id3,4 are flavor identifiers for outgoing quarks
c  gsign=+1: outgoing gluon, gsign=-1 for incoming gluon
c  colstruc =2,3 determines whether the gluon is coupled to the 
c  uppper or lower fermion line:

c  Note that colstruc is also a flag:
c  if colstruc=-1, then we are resetting.
c
c********************************************************************

      INTEGER FUNCTION FL_VVgg(iflav,colstruc)

      IMPLICIT NONE

      integer iflav(4),colstruc    ! input for color and flavor assignment

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
      
      integer lkup
      common /localkup/ lkup(numParticles,8,maxNumSubProc)
 
      integer listposition
      integer numresets
      save listposition
      save numresets

      data numresets /-1/
      data listposition /0/

      if (colstruc.eq.-1) then  ! we are resetting
c       write(*,*) "we are resetting"
        listposition=0
        numresets=numresets+1
      else if (colstruc.eq.-2) then
c       don"t do anything, just return the number of subprocesses.
      else
c increment the counter regardless of whether or not it"s the 1st time through.
        listposition=listposition+1

c       we fill info for this subprocess,if required
        if(numresets.eq.0 .and. (lha.or.hepmc)) then
          call fillColoredPartons_VVgg(iflav,
     1                                listposition+numdecay,colstruc)
        endif       ! numresets.eq.0
      endif         
      FL_VVgg=listposition
      end
c******************************************************************************
c
c   end function FL_VVgg
c
c******************************************************************************

c*****************************************************************************
c
c    begin  subroutine fillColoredPartons_VVgg
c
c*****************************************************************************
c
c       Vera Hankele, <vera@particle.uni-karlsruhe.de>
c	Initial version:  2006 March 23
c	Last modified:  2009 January 28
c
c  assigns values to the variables in the common block localHEPUP
c  in particular, this subroutine assigns values to those variables that 
c  will be stored in the lookup tables generated by writeHEPUPtable.  
c  As the name suggests, this routine only stores the information for the 
c  colored partons.  Particles without color will be dealt with in the 
c  subroutine fillColorless.
c
c*****************************************************************************

      SUBROUTINE fillColoredPartons_VVgg(iflav,listposition,colstruc)

      IMPLICIT NONE

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/process.inc"
      
      integer iflav(4),id1,id2,id3,id4,listposition,i,i1
      integer iflavour(4),colstruc
      double precision prob1, prob2, rn, RandomNumber
      logical ldebug
      parameter (ldebug=.false.)


      do i1=1,4
         iflavour(i1)=iflav(i1) 
         if(iflavour(i1).eq.0) iflavour(i1)=21   
      enddo

c...flavours of 2 quarks and 1 gluon
      id1=iflavour(1)
      id2=iflavour(2)
      id3=iflavour(3)
      id4=iflavour(4)
      
      select case (process)
      case(WPHjjLO,WMHjjLO)
         lnup(listposition)=numParticles 
      case(WPjjLO,WMjjLO)
         lnup(listposition)=7
      end select
      
      listup(1,listposition)=-1 !incoming partons
      listup(2,listposition)=-1
      listup(3,listposition)=1  !outgoing partons
      listup(4,listposition)=1  !outgoing partons
 
c...four quarks and gluon
      lidup(1,listposition)=id1    
      lidup(2,listposition)=id2
      lidup(3,listposition)=id3
      lidup(4,listposition)=id4

      do i=3,n_p
         lmothup(1,i,listposition)=1
         lmothup(2,i,listposition)=2
      enddo

      if (colstruc .gt. 0) print*, "WARNING: color not treated correctlya"
      !!!!!!!!!!!!!!!!!!!!!!!!
      ! Dummy color function !
      !!!!!!!!!!!!!!!!!!!!!!!!

      do i=1,4
         licolup(1,i,listposition)=0
         licolup(2,i,listposition)=0
      enddo
      end  ! fillColoredPartons_VVgg
c*****************************************************************************
c
c    end subroutine fillColoredPartons_VVgg
c
c*****************************************************************************
