c*****************************************************************************
c
c    begin function m2s_qqZZj2
c
c*****************************************************************************
c
c       Adapted from m2s_qqVVj2 
c
c
      real*8 function m2s_qqZZj2(
     &      bos,  !in:  XX Boson identifier,X=1 photon, X=2 Z, X=3,4 W+-, X=6 H
     &      nlo,  !in:  Born only for nlo=0, +interference for 1
     &      xi,   !in:  Feynman x parameters of incoming fermions
     &      p,    !in:  momenta associated with external fermions
     &      v,    !in:  momenta associated with gauge boson dec. fermions
     &      rn,   !in:  random number array
     &      ps_number !in:  phase-space number
     &                        )
      implicit none
c
c declare input/output variables
c
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/kkcoupl.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/lha.inc"
#include "VBFNLO/utilities/process.inc"
#include "tensor.inc"

      integer bos, nlo, ps_number
      real*8 xi(nx), p(0:3,max_p), v(0:3,n_v,11), rn(1), m2s

c  helicity selection
      INTEGER h
      COMMON /hcount / h

      complex*16 zero, z1, z2
      parameter (zero=(0d0,0d0))

c declare external functions

      real*8 dotrr
      integer FL_VVg
      external dotrr
      external FL_VVg

c declare local variables

      real*8 q_sf
      integer init/0/, I, J, mu, nu, id
      save init

c declare variables for summation process

      INTEGER ires

      integer FSIGN(n_p+n_v), gsign, physToDiag(3), nmin, nmax
      save nmax
c
c store contributions from subprocess ip in res(ip) where
      real*8 res(maxnumsubproc)
c
c declare parton distribution variables
c
      real*8 pdf(-6:6,2)
c
c define program switches
c
      logical ldebug, ldebuglt, lqiangdebug
      parameter (ldebug=.false.)          !output debug information
      PARAMETER (ldebuglt=.false.)
      parameter (lqiangdebug=.false.)      

c variables for leshouches interface
      DOUBLE PRECISION weight,rnumb,RandomNumber

cmr
      double precision  clr, xm2, xmg, b2, v2, a2
      COMMON /BKOPOU/   CLR(4,5,-1:1),XM2(6),XMG(6),B2(6,6,6),
     1                  V2(4,5),A2(4,5)
cmr

      if ( init .eq. 0 ) then

         if (bos.eq.22) then
            write(6,*) " "
            write(6,*) "ZZj amplitude square information:"
            write(6,*) "-----------------------------------------------"
            write(6,*) " "
         endif

         call printnfl(.false.)
         h=1
         init = 1
         do i = 1,maxnumsubproc
            res(i) = 0
         enddo
#ifdef WITH_FAFCDEBUG
         if (ldebuglt) then
           call fafcvars 
           call setlambda(0d0)
         endif
#endif
      else
         do i = 1,nmax
            res(i) = 0
         enddo
      endif
#ifdef WITH_FAFCDEBUG
c renormalization scale dependent quantities
c mudim might depend on ps-point
      if (ldebuglt) then
        call fafcvars 
        call setmudim(mursq(1,1))
      endif
#endif

      do mu = 0,3
         qp(mu,1) = v(mu,1,1)+v(mu,2,1)   !Z/A_e
         qm(mu,1) = v(mu,3,1)+v(mu,4,1)   !Z/A_mu
         qaa(mu,1) = qp(mu,1)+qm(mu,1)    !Z_all
      enddo
42      format( A,4(F32.16) )
      if (.false.) then
c      if (lqiangdebug) then
cmr set momenta
      p(0,1) = 2762.51701d0
      p(1,1) = 0d0
      p(2,1) = 0d0
      p(3,1) = p(0,1)
      p(0,2) = 2837.60958d0
      p(1,2) = 0d0
      p(2,2) = 0d0
      p(3,2) = -p(0,2)
      qp(0,1) = 1215.33366d0
      qp(1,1) = 359.833502d0
      qp(2,1) = 349.336111d0
      qp(3,1) = 1103.27007d0
      qm(0,1) = 1553.47925d0
      qm(1,1) = 456.808139d0
      qm(2,1) = 401.388843d0
      qm(3,1) = 1426.60293d0
C      p(0,3) = 2831.31368d0
C      p(1,3) = -816.641641d0
C      p(2,3) = -750.724955d0
C      p(3,3) = -2604.96557d0
cmr
C      p(0,1) = 56.0688444
C      p(1,1) = 0d0
C      p(2,1) = 0d0
C      p(3,1) = p(0,1)
C      p(0,2) = 3089.13295
C      p(1,2) = 0d0
C      p(2,2) = 0d0
C      p(3,2) = -p(0,2)
C      qp(0,1) = 2306.21051d0
C      qp(1,1) = -137.918263d0
C      qp(2,1) = -169.260219d0
C      qp(3,1) = -2294.04038d0
C      qm(0,1) = 349.017709d0 
C      qm(1,1) = -70.9757148d0
C      qm(2,1) = 0.170335306d0
C      qm(3,1) = -329.333573d0
C      p(0,3) = 489.973571d0
C      p(1,3) = 208.893978d0
C      p(2,3) = 169.089883d0
C      p(3,3) = -409.690149d0
cmr
C      p(0,1) = 13.1114209
C      p(1,1) = 0d0
C      p(2,1) = 0d0
C      p(3,1) = p(0,1)
C      p(0,2) = 5626.97997
C      p(1,2) = 0d0
C      p(2,2) = 0d0
C      p(3,2) = -p(0,2)
C      qp(0,1) = 640.777191
C      qp(1,1) = -68.1415515
C      qp(2,1) = -87.9492457
C      qp(3,1) = -624.421243
C      qm(0,1) = 1040.07819 
C      qm(1,1) = 44.6025449
C      qm(2,1) = -63.3395237
C      qm(3,1) = -1033.17287
C      p(0,3) = 3959.23601
C      p(1,3) = 23.5390066
C      p(2,3) = 151.288769
C      p(3,3) = -3956.27443
cmr
C      p(0,1) = 5693.51483
C      p(1,1) = 0d0
C      p(2,1) = 0d0
C      p(3,1) = p(0,1)
C      p(0,2) = 655.301502
C      p(1,2) = 0d0
C      p(2,2) = 0d0
C      p(3,2) = -p(0,2)
C      qp(0,1) = 3845.42133
C      qp(1,1) = -28.2788233
C      qp(2,1) = -268.995853
C      qp(3,1) = 3834.8131
C      qm(0,1) = 1787.31471
C      qm(1,1) = -51.245549
C      qm(2,1) = -151.07742
C      qm(3,1) = 1777.84369

cFC
cFC        p(0,1)  = 4647.14201d0 
cFC        p(1,1)  = 0.d0
cFC        P(2,1)  = 0.d0 
cFC        p(3,1)  = p(0,1)
cFC        p(0,2)  = 148.553957
cFC        p(1,2)  = 0d0
cFC        P(2,2)  = 0d0
cFC        p(3,2)  = -p(0,2)
cFC        qp(0,1) = 4247.55522d0      
cFC        qp(1,1) = 386.506646d0      
cFC        qP(2,1) = -164.143183d0
cFC        qp(3,1) = 4225.76381d0
cFC        qm(0,1) = 477.840707d0
cFC        qm(1,1) = -334.072674d0
cFC        qm(2,1) = 128.472151d0
cFC        qm(3,1) = 303.161538d0
cFC        p(0,3)  =70.3000392d0
cFC        p(1,3)  =-52.4339714d0
cFC        P(2,3)  =35.6710323d0 
cFC        p(3,3)  =-30.3372973d0

C
Cc FC
        p(0,1)  = 4240.18543d0
        p(1,1)  = 0d0
        P(2,1)  = 0d0
        p(3,1)  = p(0,1)
        p(0,2)  = 51.52056130d0
        p(1,2)  = 0d0
        P(2,2)  = 0d0
        p(3,2)  = -p(0,2)
        qp(0,1) = 2305.44984d0
        qp(1,1) = 59.3179456d0
        qP(2,1) = -12.6130442d0
        qp(3,1) = 2302.84738d0
        qm(0,1) = 1889.17067d0
        qm(1,1) = 33.965042d0
        qm(2,1) = -14.2802454d0
        qm(3,1) = 1886.60889d0
        p(0,3)  = 97.0854836d0 
        p(1,3)  =-93.2829877d0
        P(2,3)  = 26.8932896d0
        p(3,3)  =-0.791400968d0
cFC        p(0,1)  = 9.69540994d0
cFC        p(1,1)  = 0d0
cFC        P(2,1)  = 0d0
cFC        p(3,1)  = p(0,1)
cFC        p(0,2)  = 5371.09609d0
cFC        p(1,2)  = 0d0
cFC        P(2,2)  = 0d0
cFC        p(3,2)  = -p(0,2)
cFC        qp(0,1) = 2129.45075d0
cFC        qp(1,1) = -28.9592037d0
cFC        qP(2,1) = 89.1892963d0
cFC        qp(3,1) = -2125.42983d0
cFC        qm(0,1) = 2370.91819d0
cFC        qm(1,1) = -18.9288904d0
cFC        qm(2,1) = 53.368245d0
cFC        qm(3,1) = -2368.48716d0
cFC        p(0,3)  = 880.422557d0
cFC        p(1,3)  = 47.8880941d0 
cFC        P(2,3)  = -142.557541d0
cFC        p(3,3)  = -867.483692d0


cFC        p(0,1)  = 817.430524d0
cFC        p(1,1)  = 0d0
cFC        P(2,1)  = 0d0
cFC        p(3,1)  = p(0,1)
cFC        p(0,2)  = 42.8410878d0
cFC        p(1,2)  = 0d0
cFC        P(2,2)  = 0d0
cFC        p(3,2)  = -p(0,2)
cFC        qp(0,1) = 265.05705d0
cFC        qp(1,1) =29.7538264d0
cFC        qP(2,1) =103.463642d0
cFC        qp(3,1) =224.38816d0
cFC        qm(0,1) = 479.798518d0
cFC        qm(1,1) =-70.802571d0
cFC        qm(2,1) =-132.745795d0
cFC        qm(3,1) =446.382111d0
cFC        p(0,3)  = 115.416043d0
cFC        p(1,3)  = 41.0487447d0
cFC        P(2,3)  = 29.282153d0
cFC        p(3,3)  = 103.819165d0


C      p(0,3) = 716.080292 
C      p(1,3) = 79.5243725
C      p(2,3) = 420.073273
C      p(3,3) = -574.443473

cmr
C      p(0,1) = 620.5093969350863290d0
C      p(1,1) = 0d0
C      p(2,1) = 0d0
C      p(3,1) = p(0,1)
C      p(0,2) = 321.8732482260752477d0
C      p(1,2) = 0d0
C      p(2,2) = 0d0
C      p(3,2) = -p(0,2)
C      qp(0,1) = 313.7537931412830972d0
C      qp(1,1) = -261.9494670776730914d0
C      qp(2,1) = -146.1353428501501241d0
C      qp(3,1) = -12.3745559247680621d0
C      qm(0,1) = 533.3018542002153026d0
C      qm(1,1) = 335.1119673204110541d0
C      qm(2,1) = 173.8651615429903359d0
C      qm(3,1) = 365.4674969264181073d0
C      p(0,3) = 95.3269978196630774d0
C      p(1,3) = -73.1625002427380053d0
C      p(2,3) = -27.7298186928402117d0
C      p(3,3) = -54.4567922926389443d0
cmr
C      p(0,1) = 409.490614182412703421670704633296d0
C      p(1,1) = 0d0
C      p(2,1) = 0d0
C      p(3,1) = p(0,1)
C      p(0,2) = 131.662476571983915678937207009728d0
C      p(1,2) = 0d0
C      p(2,2) = 0d0
C      p(3,2) = -p(0,2)
C      qp(0,1) =  211.277011093807246313096486548452d0
C      qp(1,1) =  50.0094063239233512096940582261852d0
C      qp(2,1) =  162.922443962192074344530935374823d0
C      qp(3,1) =  85.3119777961328748988336144935825d0
C      qm(0,1) =  278.875362368736771012019167432882d0
C      qm(1,1) = -7.12589952998967724476628487429886d0
C      qm(2,1) = -189.019084317292398045541320356823d0
C      qm(3,1) =  183.513750608162997819716852029172d0
C      p(0,3) =  51.0007172918526017754922576616904d0
C      p(1,3) = -42.8835067939336739649277733518864d0
C      p(2,3) =  26.0966403551003237010103849820000d0
C      p(3,3) =  9.00240920613291502418303110081310d0
cmr
      p(0,1) = 137.571445255961d0
      p(1,1) = 0d0
      p(2,1) = 0d0
      p(3,1) = p(0,1)
      p(0,2) = 1259.23389706859d0
      p(1,2) = 0d0
      p(2,2) = 0d0
      p(3,2) = -p(0,2)
      qp(0,1) = 181.546534609122d0
      qp(1,1) = 146.975254303447d0
      qp(2,1) = -19.4595081985383d0
      qp(3,1) = 51.6097664975261d0
      qm(0,1) = 166.160924700185d0
      qm(1,1) = 18.6590193725546d0
      qm(2,1) = -3.78085356535103d0
      qm(3,1) = -137.592952747705d0
      p(0,3) = 1049.09788301524d0
      p(1,3) = -165.634273676002d0
      p(2,3) = 23.2403617638893d0
      p(3,3) = -1035.67926556245d0

      xi(1) = p(0,1)/14d3
      xi(2) = p(0,2)/14d3
cFC      p(0,3) = p(0,1)+p(0,2)-qp(0,1)-qm(0,1)
cFC      p(1,3) = p(1,1)+p(1,2)-qp(1,1)-qm(1,1)
cFC      p(2,3) = p(2,1)+p(2,2)-qp(2,1)-qm(2,1)
cFC      p(3,3) = p(3,1)+p(3,2)-qp(3,1)-qm(3,1)

      qp(0,3) = p(0,1)+p(0,2)-p(0,3)-qm(0,1)
      qp(1,3) = p(1,1)+p(1,2)-p(1,3)-qm(1,1)
      qp(2,3) = p(2,1)+p(2,2)-p(2,3)-qm(2,1)
      qp(3,3) = p(3,1)+p(3,2)-p(3,3)-qm(3,1)

cfc      print*, "third jet", p(0,3),p(1,3),p(2,3),p(3,3)
cfc      print*, "ZP", qp(0,1),qp(1,1),qp(2,1),qp(3,1)
cfc      print*, "conv0",  p(0,1)+p(0,2)-p(0,3)-qm(0,1)-qp(0,3) 
cfc      print*, "conv0",  p(1,1)+p(1,2)-p(1,3)-qm(1,1)-qp(1,3) 
cfc      print*, "conv0",  p(2,1)+p(2,2)-p(2,3)-qm(2,1)-qp(2,3) 
cfc      print*, "conv0",  p(3,1)+p(3,2)-p(3,3)-qm(3,1)-qp(3,3) 


      do mu = 0,3
         qaa(mu,1) = qp(mu,1)+qm(mu,1)    !Z_all
      enddo
      write(*,*) "PPP", 
     &  (dotrr(p(0,1),p(0,1))),
     &  (dotrr(p(0,2),p(0,2))),
     &  (dotrr(p(0,3),p(0,3))),
     &  (dotrr(qp(0,1),qp(0,1))),
     &  (dotrr(qm(0,1),qm(0,1)))
      write(*,*) "PPP", 
     &  sqrt(dotrr(p(0,1),p(0,1))),
     &  sqrt(dotrr(p(0,2),p(0,2))),
     &  sqrt(dotrr(p(0,3),p(0,3))),
     &  sqrt(dotrr(qp(0,1),qp(0,1))),
     &  sqrt(dotrr(qm(0,1),qm(0,1)))
cmr
	write(*,42) "g E1", p(0:3,1)
	write(*,42) "g E2", p(0:3,2)
	write(*,42) "Z E3", qp(0:3,1)
	write(*,42) "Z E4", qm(0:3,1)
	write(*,42) "g E5", p(0:3,3)
      endif
      qp(4,1) = dotrr(qp(0,1),qp(0,1))
      qm(4,1) = dotrr(qm(0,1),qm(0,1))
      qaa(4,1) = dotrr(qaa(0,1),qaa(0,1))

c polarisation vectors
c select helicity: h = random number for helicity (h=1:2)
      if (lqiangdebug) then
      h = mod(h+1,9) 
      ie = h/3-1
      iu = mod(h,3)-1
cFC      
c     print*,"CCC", "h", h
cFC      
      else
      h = mod(h,4) + 1
      ie = (-1)**(h+1)
      iu = (-1)**((h-1)/2)
      endif

c...Les Houches interface
        if ((lha.or.hepmc).and..not.doNLO) then
           helicity(1)=-ie
           helicity(2)= ie
           helicity(3)=-iu
           helicity(4)= iu
        endif

c for ZZ->4l precalculate leptonic tensors

      if (lqiangdebug) then
        call sxxxxx(qaa(0,1),-1,an(1,1))
        call vxxxxx(qp(0,1),ZMASS,ie,+1,wp(1,1))
        call vxxxxx(qm(0,1),ZMASS,iu,+1,wm(1,1))
        call vvhaxx(wp(1,1),wm(1,1),an(1,1),2,2,w4l(h,1))
        do mu=1,6
          atau1(mu,1) = 0d0
          atau2(mu,1) = 0d0
          wz(mu,1)    = 0d0
          wa(mu,1)    = 0d0
        enddo
      else
      CALL OXXXXX(v(0,1,1),ZERO ,-ie,+1,wve(1,1)) !W(1,3))          !e-
      CALL IXXXXX(v(0,2,1),ZERO ,+ie,-1,wep(1,1)) !W(1,4))          !e+
      CALL OXXXXX(v(0,3,1),ZERO ,-iu,+1,wmu(1,1)) !W(1,5))          !mu-
      CALL IXXXXX(v(0,4,1),ZERO ,+iu,-1,wvm(1,1)) !W(1,6))          !mu+
      CALL JIOXXX(wep(1,1),wve(1,1),GZL ,ZMASS,ZWIDTH,wp(1,1))    !Z_e
      CALL JIOXXX(wep(1,1),wve(1,1),GAL ,ZERO ,ZERO  ,atau1(1,1)) !A_e
      CALL JIOXXX(wvm(1,1),wmu(1,1),GZL ,ZMASS,ZWIDTH,wm(1,1))    !Z_mu
      CALL JIOXXX(wvm(1,1),wmu(1,1),GAL ,ZERO ,ZERO  ,atau2(1,1)) !A_mu
      call zz_ztozz(v(0,1,1),1,wz(1,1),0) !Z -> e+ e- mu+ mu-
      call zz_atozz(v(0,1,1),1,wa(1,1),0) !A -> e+ e- mu+ mu-
      if ((nlo .eq. -8) .or. (nlo.eq.1)) 
     &  call zz_htozz(v(0,1,1),1,w4l(h,1),0) !H -> e+ e- mu+ mu-
      if (narrowwidth.ne.0) then
c no gamma* contributions
        do mu=1,6
          atau1(mu,1) = 0d0
          atau2(mu,1) = 0d0
        enddo
      endif
      endif !lqiangdebug

c if desired, print output information

      if ( ldebug ) then
 10      format( " p(", i1, ") = ", 4(f10.3, 2x) )
 20      format( " v(", i1, ") = ", 4(f10.3, 2x) )
c
         write(6,*) " "
         do i = 1, n_v
            write(6,20) i, v(0,i,1), v(1,i,1), v(2,i,1), v(3,i,1)
         end do
         print*," parton momenta "
         do i = 1, n_p-1
            write(6,10) i, p(0,i), p(1,i), p(2,i), p(3,i)
         end do
c
         write(6,*) " "
         write(6,*) "xi(1), xi(2) =", xi
         write(6,*) "mu_f =",sqrt(mufsq(1,1))
      end if
c
c call PDF subroutine in order to determine parton
c distributions in the incoming (anti)protons.
c
      q_sf = sqrt(mufsq(1,1))

      call pdfproton( xi(1), q_sf, pdf(-6,1) )        ! f_a(x1)
      call pdfproton( xi(2), q_sf, pdf(-6,2) )        ! f_a(x2)


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

C     Now sum over the subprocesses contributing to ZZj production

      nmax = FL_VVg(fsign,-1)          !reset counter for subprocesses to 0

C*******************  q1 qbar2 ---> Z Z g   **********************
         
c   physToDiag(ext.momentum label) = Feynman diagram label

      physToDiag(1)=1    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=2
      physToDiag(3)=3    ! gluon

      fsign(1) = 1          !incoming quark
      fsign(2) = -1         !incoming antiquark
      fsign(3) = 1          !outgoing neutrino
      fsign(4) = -1         !outgoing lepton
      fsign(5) = 1          !outgoing antilepton
      fsign(6) = -1         !outgoing antineutrino
      gsign    = 1          !outgoing gluon

      call qq_ZZj2(xi,p,v,physToDiag,fsign,gsign,bos, nlo,
     &              pdf,res,nmin,nmax,ps_number)

      if (nlo .ne. -8) then
cc gg is symmetric -> would be double-counting
C*******************  qbar1 q2 ---> Z Z g   **********************

      physToDiag(1)=2    !physToDiag(1/2) are labels of incoming quarks
      physToDiag(2)=1
      physToDiag(3)=3    ! gluon

      fsign(1) = 1
      fsign(2) = -1

      call qq_ZZj2(xi,p,v,physToDiag,fsign,gsign,bos, nlo,
     &              pdf,res,nmin,nmax,ps_number)

C*******************  g q1 ---> Z Z q2  ***********************************

      physToDiag(1)=3
      physToDiag(2)=1
      physToDiag(3)=2

      fsign(1) = 1
      fsign(2) = 1
      gsign    = -1

      call qq_ZZj2(xi,p,v,physToDiag,fsign,gsign,bos, nlo,
     &              pdf,res,nmin,nmax,ps_number)

C*******************  qbar2 g ---> Z Z qbar1   **********************

      physToDiag(1)=2
      physToDiag(2)=3
      physToDiag(3)=1

      fsign(1) = -1
      fsign(2) = -1
      gsign = -1

      call qq_ZZj2(xi,p,v,physToDiag,fsign,gsign,bos, nlo,
     &              pdf,res,nmin,nmax,ps_number)

C*******************  g q2bar ---> Z Z q1bar  **********************

      physToDiag(1)=3
      physToDiag(2)=2
      physToDiag(3)=1

      fsign(1) = -1
      fsign(2) = -1
      gsign = -1

      call qq_ZZj2(xi,p,v,physToDiag,fsign,gsign,bos, nlo,
     &              pdf,res,nmin,nmax,ps_number)

C*******************  q1 g ---> Z Z q2   **********************

      physToDiag(1)=1
      physToDiag(2)=3
      physToDiag(3)=2

      fsign(1) = 1
      fsign(2) = 1

      call qq_ZZj2(xi,p,v,physToDiag,fsign,gsign,bos, nlo,
     &              pdf,res,nmin,nmax,ps_number)

      endif
c*****************  end of process evaluation  *********************

      if (init.eq.1) then
         init = init+1
      endif

      m2s = 0
      DO IRES = 1,NMAX
         m2s = m2s + RES(IRES)
      ENDDO

c...Les Houches interface - the most propable subprocess   
      if ((lha.or.hepmc).and..not.doNLO) then
         i=0
         weight=0.d0
         rnumb=RandomNumber()
         do while((i.le.nmax).and.(weight.le.rnumb*m2s))
            i=i+1
            weight=weight+res(i)
            iprocess=i
         enddo
      endif

      if (lqiangdebug) then
        m2s = m2s * 9d0/2d0 * (16d0*pi/xmg(2))**2     ! factor 4 for random helicity summation
      else
        m2s = m2s * 4d0      ! factor 4 for random helicity summation
      endif

      m2s_qqZZj2 = m2s
      
      RETURN
      END

c*****************************************************************************
c
c    end function m2s_qqZZj2
c
c*****************************************************************************

c******************************************************************
c
c   begin subroutine qq_ZZj2
c
c*****************************************************************

      subroutine qq_ZZj2(xi,p,v,physToDiag,fsign,gsign, bos, nlo,
     1                  pdf,res,nmin,nmax,ps_number)
      implicit none

#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/lha.inc"

      real*8 p(0:3,max_p-1), v(0:3,n_v), xi(nx)
      real*8 pdf(-6:6,2), res(maxnumsubproc)
      integer physToDiag(3), fsign(n_p+n_v), gsign, 
     &        bos, nlo, nmin, nmax, ps_number
c
c qq_ZZj2 calls the amplitude square routines 
c             qqZZj    for qq --> ZZ --> 4 leptons
c
c  INPUT:  p(0:3,3)        external physical parton momenta
c          v(0:3,nv)       decay momenta
c                          the sum q(mu) = v(mu,1)+...+v(mu,nv) is needed
c          physToDiag(3)   physToDiag(ext.mom. label) = Feynman diagram label
c          fsign           sign factors for amplitude calls; see qqZqq
c          nlo             nlo = 0: calculate LO only
c                          nlo = 1: calculate full NLO (Born + interference)
c  OUTPUT:
c          uuvv(ID)   output in format as in qqZqq(j), but corresponding
c          etc.            to res(*) in m2s_qqZqq
c     or   res(k)   
c
c  In and Output
c    nmin, nmax            range of process numbers for this call
c
      double precision N                              ! color factors
      parameter(N=3d0)
c
c alfas, scales etc
#include "VBFNLO/utilities/scales.inc"
#include "tensor.inc"
#include "VBFNLO/utilities/coupl.inc"

c local variables
      real*8 uuzz, ddzz, uuzz2, ddzz2
      real*8 pbar(0:3,n_p+n_v), qbar(0:4), dotrr
      real*8 polcolq, polcolg, polcolgg
#ifdef WITH_FAFCDEBUG
      double precision pfafc(2,5) 
#endif
      double precision NCmatrixelt(0:1,0:1)
      double complex zero
      integer iflav(4), diagToPhys(3), FL_VVg
      external dotrr, FL_VVg

      logical ChargedCurrent, sametype, oneAntiparticle
      logical ldebug, ldebuglt
      parameter (ldebug=.false.)
      parameter (ldebuglt=.false.)
      parameter (zero = (0d0,0d0) )
      integer i,if1,if2,icc1,j,k,mu

      uuzz = 0d0
      uuzz2 = 0d0
      ddzz = 0d0
      ddzz2 = 0d0

      nmin = nmax+1
      do i = 1,n_p
         diagToPhys(physToDiag(i)) = i
      enddo

      polcolq = 1d0/(4d0*N**2*xi(1)*xi(2))
      polcolg = 1d0/(4d0*N*(N**2-1)*xi(1)*xi(2))
      polcolgg= 1d0/(4d0*(N**2-1)**2*xi(1)*xi(2))

      do mu = 0,3
         do i = 1,n_p
            pbar(mu,physToDiag(i))=p(mu,i)
         enddo
         qbar(mu) = pbar(mu,3)
      enddo


      do mu = 0,3             ! kinematics for WW --> 4 lepton decay
         pbar(mu,3) = v(mu,1) ! ve
         pbar(mu,4) = v(mu,2) ! e+
         pbar(mu,5) = v(mu,3) ! mu-
         pbar(mu,6) = v(mu,4) ! nu_mu~
         pbar(mu,7) = qbar(mu)
      enddo
      fsign(7) = gsign

      if ((nlo .eq. -8).or.(
     &  (nlo.eq.1).and.(
     &    (physToDiag(1).eq.1) .and. (physToDiag(2).eq.2)
     & ))) then
c gluon-fusion loop contributions
#ifdef WITH_FAFCDEBUG
      if (ldebuglt) then
      do i = 1,2             
        pfafc(1,i) = 0d0!sqrt(abs( dotrr(pbar(0,i),pbar(0,i)) ))
        pfafc(2,i) = sqrt( pbar(0,i)**2 - pfafc(1,i)**2 )
        if ((pbar(1,i)/pfafc(2,i)).ge.0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      sqrt(1-(pbar(2,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(2,i)/pfafc(2,i),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(1,i)/pfafc(2,i)).le.-0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      -sqrt(1-(pbar(2,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(2,i)/pfafc(2,i),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(2,i)/pfafc(2,i)).ge.0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(2,i)/pfafc(2,i)).le.-0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      -sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(3,i)/pfafc(2,i)).ge.0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      pbar(2,i)/pfafc(2,i),
     &      sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(2,i)/pfafc(2,i))**2))
        else if ((pbar(3,i)/pfafc(2,i)).le.-0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      pbar(2,i)/pfafc(2,i),
     &      -sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(2,i)/pfafc(2,i))**2))
        else
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      pbar(2,i)/pfafc(2,i),
     &      pbar(3,i)/pfafc(2,i) )
        endif
      enddo
        pfafc(1,5) = 0d0!sqrt(abs( dotrr(pbar(0,i),pbar(0,i)) ))
        pfafc(2,5) = sqrt( pbar(0,7)**2 - pfafc(1,5)**2 )
        if ((pbar(1,7)/pfafc(2,5)).ge.0.995d0) then
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      sqrt(1-(pbar(2,7)/pfafc(2,5))**2-(pbar(3,7)/pfafc(2,5))**2),
     &      pbar(2,7)/pfafc(2,5),
     &      pbar(3,7)/pfafc(2,5) )
        else if ((pbar(1,7)/pfafc(2,5)).le.-0.995d0) then
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      -sqrt(1-(pbar(2,7)/pfafc(2,5))**2-(pbar(3,7)/pfafc(2,5))**2),
     &      pbar(2,7)/pfafc(2,5),
     &      pbar(3,7)/pfafc(2,5) )
        else if ((pbar(2,7)/pfafc(2,5)).ge.0.995d0) then
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      pbar(1,7)/pfafc(2,5),
     &      sqrt(1-(pbar(1,7)/pfafc(2,5))**2-(pbar(3,7)/pfafc(2,5))**2),
     &      pbar(3,7)/pfafc(2,5) )
        else if ((pbar(2,7)/pfafc(2,5)).le.-0.995d0) then
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      pbar(1,7)/pfafc(2,5),
     &      -sqrt(1-(pbar(1,7)/pfafc(2,5))**2-(pbar(3,7)/pfafc(2,5))**2),
     &      pbar(3,7)/pfafc(2,5) )
        else if ((pbar(3,7)/pfafc(2,5)).ge.0.995d0) then
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      pbar(1,7)/pfafc(2,5),
     &      pbar(2,7)/pfafc(2,5),
     &      sqrt(1-(pbar(1,7)/pfafc(2,5))**2-(pbar(2,7)/pfafc(2,5))**2))
        else if ((pbar(3,7)/pfafc(2,5)).le.-0.995d0) then
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      pbar(1,7)/pfafc(2,5),
     &      pbar(2,7)/pfafc(2,5),
     &      -sqrt(1-(pbar(1,7)/pfafc(2,5))**2-(pbar(2,7)/pfafc(2,5))**2))
        else
          call VecSet(5,pfafc(1,5),pfafc(2,5),
     &      pbar(1,7)/pfafc(2,5),
     &      pbar(2,7)/pfafc(2,5),
     &      pbar(3,7)/pfafc(2,5) )
        endif
        pfafc(1,3) = sqrt(abs( dotrr(qp(0,1),qp(0,1)) ))
        pfafc(2,3) = sqrt( qp(0,1)**2 - pfafc(1,3)**2 )
        call VecSet(3,pfafc(1,3),pfafc(2,3),
     &    qp(1,1)/pfafc(2,3),
     &    qp(2,1)/pfafc(2,3),
     &    qp(3,1)/pfafc(2,3) )
        pfafc(1,4) = sqrt(abs( dotrr(qm(0,1),qm(0,1)) ))
        pfafc(2,4) = sqrt( qm(0,1)**2 - pfafc(1,4)**2 )
        call VecSet(4,pfafc(1,4),pfafc(2,4),
     &    qm(1,1)/pfafc(2,4),
     &    qm(2,1)/pfafc(2,4),
     &    qm(3,1)/pfafc(2,4) )
      endif
#endif
c instead of anti-fermion here initial/final
        fsign(2)=-fsign(2)
        call GGZZg(pbar,fsign,qbar,gsign,bos,nlo,1,uuzz)
c switch back for nlo.eq.1
        fsign(2)=-fsign(2)
        iflav(1)=0
        iflav(2)=0
        k=FL_VVg(iflav,1)
  
        res(k)=pdf(sign1*iflav(1),1)
     &        *pdf(sign2*iflav(2),2)
     &        *uuzz*polcolgg

      endif
      if (nlo .ne. -8) then

#ifdef WITH_MADGRAPH
        if(ldebug) then
          call suu_zzg(pbar,fsign,uuzz2)
          call sdd_zzg(pbar,fsign,ddzz2)
        endif
#endif
#ifdef WITH_FAFCDEBUG
      do i = 1,7             
        pfafc(1,i) = 0d0!sqrt(abs( dotrr(pbar(0,i),pbar(0,i)) ))
        pfafc(2,i) = sqrt( pbar(0,i)**2 - pfafc(1,i)**2 )
        if ((pbar(1,i)/pfafc(2,i)).ge.0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      sqrt(1-(pbar(2,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(2,i)/pfafc(2,i),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(1,i)/pfafc(2,i)).le.-0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      -sqrt(1-(pbar(2,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(2,i)/pfafc(2,i),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(2,i)/pfafc(2,i)).ge.0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(2,i)/pfafc(2,i)).le.-0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      -sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(3,i)/pfafc(2,i))**2),
     &      pbar(3,i)/pfafc(2,i) )
        else if ((pbar(3,i)/pfafc(2,i)).ge.0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      pbar(2,i)/pfafc(2,i),
     &      sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(2,i)/pfafc(2,i))**2))
        else if ((pbar(3,i)/pfafc(2,i)).le.-0.995d0) then
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      pbar(2,i)/pfafc(2,i),
     &      -sqrt(1-(pbar(1,i)/pfafc(2,i))**2-(pbar(2,i)/pfafc(2,i))**2))
        else
          call VecSet(i,pfafc(1,i),pfafc(2,i),
     &      pbar(1,i)/pfafc(2,i),
     &      pbar(2,i)/pfafc(2,i),
     &      pbar(3,i)/pfafc(2,i) )
        endif
      enddo
#endif
      
        call QQZZj(pbar,fsign,qbar,gsign,nlo,1,uuzz,ddzz,ps_number)

#ifdef WITH_MADGRAPH
        if (ldebug) then
           if(abs(uuzz/uuzz2 - 1d0).ge.1.0d-5) then
              print*,"uuzz =   ",uuzz
              print*,"uuzz2 =  ",uuzz2
           endif
           if(abs(ddzz/ddzz2 - 1d0).ge.1.0d-5) then
              print*,"ddzz =   ",ddzz
              print*,"ddzz2 =  ",ddzz2
           endif
        endif
#endif

        NCmatrixelt(0,0)=uuzz
        NCmatrixelt(1,1)=ddzz


        if (gsign .eq. 1) then
           iflav(3) = 0           ! final state gluon id
           do if1=1,nfl
              iflav(1)=if1*fsign(physToDiag(1))
              iflav(2)=if1*fsign(physToDiag(2))
                 k=FL_VVg(iflav,1)
                 res(k)=pdf(sign1*iflav(1),1)
     &                 *pdf(sign2*iflav(2),2)
     &                 *NCmatrixelt(mod(if1,2),mod(if1,2))*polcolq
           enddo
        elseif (gsign.eq.-1) then !initial gluon section
           j = 0
           if (physToDiag(1).eq.3) then
              j=2              ! j=2: emission off 1,2 line
           elseif (physToDiag(2).eq.3) then
              j=3              ! j=3: emission off 3,4 line
           endif
           do if1=1,nfl
              iflav(j-1) = 0
              iflav(4-j)=if1*fsign(physToDiag(4-j))
              iflav(3)  =if1*fsign(physToDiag(3))
              k=FL_VVg(iflav,j)
              res(k)=pdf(sign1*iflav(1),1)
     &              *pdf(sign2*iflav(2),2)
     &              *NCmatrixelt(mod(if1,2),mod(if1,2))*polcolg
           enddo
        endif 
      endif

      nmax = FL_VVg(iflav,-2)

      end
c******************************************************************
c
c   end subroutine qq_ZZj2
c
c*****************************************************************
