***************************************************************************
*
*  This is the REPOLO main program. 
*
****************************************************************************

****************************************************************************
*  Last modified: 03.05.2011
****************************************************************************

      subroutine dorepolo
      implicit none


c include global variables via include-files, e.g. "procID"
#include "VBFNLO/utilities/global.inc"
#include "VBFNLO/utilities/tau_hel.inc"
#include "VBFNLO/utilities/process.inc"
#include "VBFNLO/utilities/scales.inc"
#include "VBFNLO/utilities/koppln.inc"

c define local variables
c--------------------------------------------------------
c     variables for the integration loops
      integer ps_number
      integer iteration, seed
      integer*8 Ncall, i, two
      parameter (two = 2)
      common/Ncall1/Ncall,iteration

c     digit characters for numbering the grid output file
      character*1 digit(61)
      data digit /'1','2','3','4','5','6','7','8','9','A',
     1            'B','C','D','E','F','G','H','I','J','K',
     1            'L','M','N','O','P','Q','R','S','T','U',
     1            'V','W','X','Y','Z','a','b','c','d','e',
     1            'f','g','h','i','j','k','l','m','n','o',
     1            'p','q','r','s','t','u','v','w','x','y',
     1            'z'/
c     the filenames for the grid output
      character*250 file_name_out, file_name_in

c     random number array controlled by monaco
      real*8 rand(max_PS_dim),rn
      real*8 x(nx)   ! feynman x parameters
c     4-momenta of the partons involved in the basic QCD 2->2(+1)+X process.
c     The partons are the incoming quarks/gluons + the 2 tagging jets + 
c     the real emission (for NLO only)
      real*8 p(0:3,max_p,max_kin) 
      real*8 xuz(2,2:max_kin)
c     Additional particles like a higgs boson, or any decay products 
c     (also hadronic ones). in short: everything which is not a parton 
c     involved in the basic QCD 2->2(+1)+X process.
      real*8 v(0:3,max_v,max_kin) 

c     Jets after jet-defining function
      real*8 jets(0:7,max_jets,max_kin) 
      integer n_jets(max_kin)
c     Charged leptons, i.e. electrons, muons, taus but not neutrinos
      real*8 leptons(0:8,max_v,max_kin)
      integer n_leptons(max_kin)
c     photons
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin)
c     invisible particles, i.e. neutrinos, LSP
      real*8 invisible(0:8,max_v,max_kin)
      integer n_invisible(max_kin)          
      
c     event weights, jacobi-factors, amplitude-squares
      real*8 weight, dps, m2s(0:max_kin), dsig(0:max_kin), q12, q34

c     read in variables
      character*20 text
      real*8 unimportant

      logical bornlike
      common/gluon/ bornlike
c     external functions
      logical Cuts       ! general cut-function
      logical Choose_PS  ! general PS-function
      real*8 amplitude   ! general squared-amplitude function
      external Cuts, amplitude, Choose_PS

      real*8 RandomNumber,bornflav, realflav
      external RandomNumber     

      integer flavcombi
ccc      integer counter, counterSM, counterBSM
      common/flavREW/bornflav, realflav, flavcombi   
** Functions and variables to fully define the input gridname
      character*250 GetInputPath
      character*250 path
      external GetInputPath

c     debugging and testing variables and switches
      logical showtiming
      parameter (showtiming=.true.)

      logical lokt(max_kin)
      
      double precision m2sSpin2

      real*8 dotrr
      external dotrr
      logical Finalprinting
      parameter(Finalprinting=.true.)
      
      logical rewSpin2
      common/rewSpin2/rewSpin2

      character*250 inputfile
      common/RepoloInput/inputfile
           
      integer mu

      call PrintVBFNLOVersion(3)

** LoopTools initialisation
#ifdef WITH_LT
      call initLT      
#endif


      if (showtiming) call mtime

      write(*,*) '-----------------------------------------------------'
      write(*,*) '-----------      REPOLO  INFO       -----------------'
      write(*,*) '-----------------------------------------------------'
      write(*,*) 'This is the first run within the SM, output is'
      write(*,*) 'written to rew_sm.dat'

c     first, do all the initialization
    
      call InitGlobalParameters_REPOLO
      

      if(procID.eq.aajj) then
          rewSpin2=.true.
          procID=hjj_aa
          process=hjj_aa
      endif      

      call symmetry_factor

      call PrintProcInfo
      
      call InitCouplings

      call InitProcess
      

** Setting input path
      path = GetInputPath()
      inputfile = path(1:len_trim(path))//"/"//inputfile
      
ccc      counter=0
ccc      counterSM=0
ccc      counterBSM=0
         

      open(unit=333,file=inputfile(1:len_trim(inputfile)),err=999)
      open(unit=334,file='rew_sm.dat',err=999) 
      
      
      read(333,fmt='(a)') text      
      do while (text(1:6).ne.'</init')
           read(333,fmt='(a)') text    
      enddo
cccc FORCE to calculate within the SM!! 2 different input files???      
      read(333,fmt='(a)') text  
      
      do while (text(1:6).eq.'<event')
        
         call read_pwg(p(0,1,1), v(0,1,1),bornlike )

         call defineJets(p, v, 1, jets, n_jets)
         call DefineLeptons(v, 1, leptons, n_leptons)
         call DefineInvisible(v,1, invisible, n_invisible)
         call DefinePhotons(v, 1, photons, n_photons)

         call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 0)

         if (bornlike) then
            m2s(0) =  Amplitude(rand, p, x, v, 0, ps_number)  
            m2s(0) = symmetryfactor * m2s(0)            
         else
            call RE_Amplitude(rand, 
     -                      p, x, v, lokt, xuz, m2s,ps_number) 
            m2s(0) = symmetryfactor * m2s(0)
         endif
ccc         if((m2s(0)/(BHGAM*xgh)*xmh**2).lt.1d-11) then
ccc            counter=counter+1
ccc         endif
         write(334,*) m2s(0)
ccc         counterSM=counterSM+1
         read(333,fmt='(a)') text             
      enddo

      close(333)
      close(334)


ccc      write(*,*) " Unstable events: ",counter," = ", real(counter)/real(counterSM)*100d0," %"
ccc      write(*,*) ''
ccc      if (real(counter)/real(counterSM).gt.0.001)  write(*,*) 'WARNING: YOU ARE LOOSING TO MANY EVENTS'
      
cccc second time: spin2 or anamolous couplings!          
      
      
      write(*,*) ''
      write(*,*) '-----------------------------------------------------'
      write(*,*) 'This is the 2nd run with BSM physics, output is'
      write(*,*) 'written to rew_bsm.dat'
      
      
      if(rewSpin2) then
          procID=aajj
          process=aajj
      endif

      call symmetry_factor

      call InitCouplings   
   
      call InitProcess
      
      open(unit=333,file=inputfile(1:len_trim(inputfile)),err=999)
      open(unit=335,file='rew_bsm.dat',err=999)          
      read(333,fmt='(a)') text      
      do while (text(1:6).ne.'</init')
           read(333,fmt='(a)') text    
      enddo
cccc FORCE to calculate within the SM!! 2 different input files???      
      read(333,fmt='(a)') text  
      
      do while (text(1:6).eq.'<event')
      
         call read_pwg(p(0,1,1), v(0,1,1),bornlike )
         call defineJets(p, v, 1, jets, n_jets)
         call DefineLeptons(v, 1, leptons, n_leptons)
         call DefineInvisible(v,1, invisible, n_invisible)
         call DefinePhotons(v, 1, photons, n_photons)      
      
         call Scales(p, v, jets, n_jets, leptons, 
     1                          n_leptons, photons, n_photons, 1, 0)
     
         if(process.eq.aajj) then

            if (bornlike) then
            
               m2sSpin2=0d0
               m2s(0) =  Amplitude(rand, p, x, v, 0, ps_number)  !real helicity summation
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor

               m2s(0) =  Amplitude(rand, p, x, v, 0, ps_number)
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor
               
               m2s(0) =  Amplitude(rand, p, x, v, 0, ps_number)
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor
   
               m2s(0) =  Amplitude(rand, p, x, v, 0, ps_number)
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor

               m2s(0)=m2sSpin2
            else
            
               m2sSpin2=0d0
               call RE_Amplitude(rand, 
     -                          p, x, v, lokt, xuz, m2s,ps_number)     !real helicity summation
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor

               call RE_Amplitude(rand, 
     -                          p, x, v, lokt, xuz, m2s,ps_number)   
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor
               
               call RE_Amplitude(rand, 
     -                          p, x, v, lokt, xuz, m2s,ps_number)   
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor
   
               call RE_Amplitude(rand, 
     -                          p, x, v, lokt, xuz, m2s,ps_number)   
               m2sSpin2 = m2sSpin2+ m2s(0)/4d0*symmetryfactor         
               
               m2s(0)=m2sSpin2
            endif

         else
     
            if (bornlike) then
               m2s(0) =  Amplitude(rand, p, x, v, 0, ps_number)
               m2s(0) = symmetryfactor * m2s(0)

            else
               call RE_Amplitude(rand, 
     -                          p, x, v, lokt, xuz, m2s,ps_number)      
               m2s(0) = symmetryfactor * m2s(0)
            endif
         endif
      
         write(335,*) m2s(0)

ccc         counterBSM=CounterBSM+1
         read(333,fmt='(a)') text  
      enddo
      

      close(333)      
      close(335)

      if (showtiming) call mtime
      
ccc      if (counterSM.ne.counterBSM) print*, "ERROR!!!!  #SM-events != #BSM-events ", counterSM, counterBSM
   
      STOP
999   continue
      write(*,*) 'Error while reading file pwgevents2.lhe. STOP!'
      write(*,*) i
      stop

      END
c     END OF VBFNLO
    


